#include "GLShader.h"
#include "Base.h"
#include <iostream>
#include <fstream>
#include <sstream>
Shader::Shader(const char* vertexPath, const char* fragmentPath)
{
           // 1. retrieve the vertex/fragment source code from filePath
        std::string vertexCode;
        std::string fragmentCode;
        std::ifstream vShaderFile;
        std::ifstream fShaderFile;
        // ensure ifstream objects can throw exceptions:
        vShaderFile.exceptions (std::ifstream::failbit | std::ifstream::badbit);
        fShaderFile.exceptions (std::ifstream::failbit | std::ifstream::badbit);
        try 
        {
            // open files
            vShaderFile.open(vertexPath);
            fShaderFile.open(fragmentPath);
            std::stringstream vShaderStream, fShaderStream;
            // read file's buffer contents into streams
            vShaderStream << vShaderFile.rdbuf();
            fShaderStream << fShaderFile.rdbuf();		
            // close file handlers
            vShaderFile.close();
            fShaderFile.close();
            // convert stream into string
            vertexCode = vShaderStream.str();
            fragmentCode = fShaderStream.str();			
        }
        catch (std::ifstream::failure& e)
        {
            std::cout << "ERROR::SHADER::FILE_NOT_SUCCESFULLY_READ: " << e.what() << std::endl;
        }
        const char* vShaderCode = vertexCode.c_str();
        const char * fShaderCode = fragmentCode.c_str();
        // 2. compile shaders
        unsigned int vertex, fragment;
        // vertex shader
        vertex = glCreateShader(GL_VERTEX_SHADER);
        glShaderSource(vertex, 1, &vShaderCode, NULL);
        glCompileShader(vertex);
        checkCompileErrors(vertex, "VERTEX");
        // fragment Shader
        fragment = glCreateShader(GL_FRAGMENT_SHADER);
        glShaderSource(fragment, 1, &fShaderCode, NULL);
        glCompileShader(fragment);
        checkCompileErrors(fragment, "FRAGMENT");
        // shader Program
        program = glCreateProgram();
        glAttachShader(program, vertex);
        glAttachShader(program, fragment);
        glLinkProgram(program);
        checkCompileErrors(program, "PROGRAM");
        // delete the shaders as they're linked into our program now and no longer necessery
        glDeleteShader(vertex);
        glDeleteShader(fragment);

}

void Shader::bind(){
    GLCALL(glUseProgram(program));
}

void Shader::unbind(){
    GLCALL(glUseProgram(0));
}

void Shader::setuniform(const GLchar* uName, unsigned int value){
    GLCALL(glUniform1i(glGetUniformLocation(program, uName), value));
}


void Shader::setuniform(const GLchar* uName, int value){
    GLCALL(glUniform1i(glGetUniformLocation(program, uName), value));
}

void Shader::setuniform(const GLchar* uName, GLfloat value){
    GLCALL(glUniform1i(glGetUniformLocation(program, uName), value));
}

void Shader::setuniform(const GLchar* uName, GLfloat x, GLfloat y){
    GLCALL(glUniform2f(glGetUniformLocation(program, uName), x, y));
}

void Shader::setuniform(const GLchar* uName, GLfloat x, GLfloat y, GLfloat z){
    GLCALL(glUniform3f(glGetUniformLocation(program, uName), x, y, z));
}    

void Shader::setuniform(const GLchar* uName, glm::vec3 vector){
    GLCALL(glUniform3f(glGetUniformLocation(program, uName), vector.x, vector.y, vector.z));
}

void Shader::setuniform(const GLchar* uName, GLfloat x, GLfloat y, GLfloat z, GLfloat w){
    GLCALL(glUniform4f(glGetUniformLocation(program, uName), x, y, z, w));
}

void Shader::setuniform(const GLchar* name, const glm::mat4 &mat)
{
    glUniformMatrix4fv(glGetUniformLocation(program, name), 1, GL_FALSE, &mat[0][0]);
}

void Shader::setuniform(const GLchar* uName, glm::vec2 vector){
    GLCALL(glUniform2f(glGetUniformLocation(program, uName), vector.x, vector.y));
}

void Shader::setuniform(const GLchar* uName, glm::vec4 vector){
    GLCALL(glUniform4f(glGetUniformLocation(program, uName), vector.x, vector.y, vector.z, vector.w));
}
void Shader::setuniform(const GLchar* uName, GLuint tex2d, GLint unit){ // sample 2d
    GLCALL(glActiveTexture(GL_TEXTURE0 + unit));
    GLCALL(glBindTexture(GL_TEXTURE_2D, tex2d));
    GLCALL(glUniform1i(glGetUniformLocation(program, uName), unit));
}